/*
 * pdscvector.h
 * 
 * Copyright 2011 Fernando Pujaico Rivera <fernando.pujaico.rivera@gmail.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 */

/** \file pdscvector.h
 *  \author Fernando Pujaico Rivera
 *  \date 10-05-2011
 *  \brief Funciones que trabajan con vectores complejos.
 *  
 *  Estas funciones trabajan con vectores complejos de la forma.<br>
 *  \image html pdscvector.png "Vector de Nel elementos"
 *  \b Nel es el número de elementos.
 */

#ifndef __PDSCVECTOR_H__
#define __PDSCVECTOR_H__

#ifdef __cplusplus
extern "C" {
#endif 


#ifndef TRUE
	#define TRUE 1
#endif

#ifndef FALSE
	#define FALSE 0
#endif

#include <stdio.h>
#include <pds/pdscaglobal.h>
#include <pds/pdscn.h>
#include <pds/pdsra.h>

/** \defgroup PdsCVectorGroup Funciones del módulo PdsCVector.
 * @{
 */


/*! \struct PdsCVector
 *  \brief La estructura tipo  PdsCVector .
 *  Esta estructura genera un vector de Nel elementos.
 *  Para usar incluir pds/pdsca.h.<br>
 *  \image html pdscvector.png "Vector de Nel elementos"
 *  \b Nel es el número de elementos.
 *  \ingroup PdsCVectorGroup
 *  \author Fernando Pujaico Rivera
 */
typedef struct 
{
	/*! Un arreglo de Nel elementos complejos. */
	PdsComplex *V;
	/*! Número de elementos. */
	PdsCaNatural Nel;
}PdsCVector;


/** \fn PdsCVector *pds_cvector_new(PdsCaNatural Nel)
 *  \brief Crea un vector de tipo PdsCVector e inicia con cero todos los elementos.
 *  \param[in] Nel Es el número de elementos del vector.
 *  \return Un puntero al vector de tipo PdsCVector.
 *  \ingroup PdsCVectorGroup
 */
PdsCVector *pds_cvector_new(PdsCaNatural Nel);


/** \fn PdsCVector *pds_cvector_new_cvector(const PdsCVector *VecSrc)
 *  \brief Crea un vector de tipo PdsCVector usando como fuente el vector VecSrc.
 *  \param[in] VecSrc Es el vector imagen que se usará como fuente.
 *  \return Un puntero al vector de tipo PdsCVector, retorna NULL en caso de error.
 *  \ingroup PdsCVectorGroup
 */
PdsCVector *pds_cvector_new_cvector(const PdsCVector *VecSrc);


/** \fn PdsCVector *pds_cvector_new_vector(const PdsVector *VecSrc)
 *  \brief Crea un vector de tipo PdsCVector usando como fuente el vector VecSrc
 *  y lo copia en la parte real del vector creado.
 *  \param[in] VecSrc Es el vector imagen que se usará como fuente.
 *  \return Un puntero al vector de tipo PdsCVector, retorna NULL en caso de error.
 *  \ingroup PdsCVectorGroup
 */
PdsCVector *pds_cvector_new_vector(const PdsVector *VecSrc);


/** \fn int pds_cvector_init_complex(PdsCVector *Vector,PdsComplex Val)
 *  \brief Inicia el vector de tipo puntero PdsCVector. <br><b>Vector=Val</b>.
 *  \param[in,out] Vector El vector a iniciar.
 *  \param[in] Val Es el valor complejo inicial de los elementos.
 *  \return TRUE si todo fue bien o FALSE si no.
 *  \ingroup PdsCVectorGroup
 */
int pds_cvector_init_complex(PdsCVector *Vector,PdsComplex Val);


/** \fn int pds_cvector_init_real(PdsCVector *Vector,PdsCaReal Val)
 *  \brief Inicia la parte real del vector de tipo puntero PdsCVector. <br><b>Vector.Real=Val</b>.
 *  \param[in,out] Vector El vector a iniciar.
 *  \param[in] Val Es el valor real inicial de los elementos.
 *  \return TRUE si todo fue bien o FALSE si no.
 *  \ingroup PdsCVectorGroup
 */
int pds_cvector_init_real(PdsCVector *Vector,PdsCaReal Val);


/** \fn int pds_cvector_init_imag(PdsCVector *Vector,PdsCaReal Val)
 *  \brief Inicia la parte imaginaria del vector de tipo puntero PdsCVector. <br><b>Vector.Imag=Val</b>.
 *  \param[in,out] Vector El vector a iniciar.
 *  \param[in] Val Es el valor imaginario inicial de los elementos.
 *  \return TRUE si todo fue bien o FALSE si no.
 *  \ingroup PdsCVectorGroup
 */
int pds_cvector_init_imag(PdsCVector *Vector,PdsCaReal Val);


/** \fn int pds_cvector_init_cvector(PdsCVector *Vector,const PdsCVector *VecSrc)
 *  \brief Inicia el vector de tipo puntero PdsCVector con otro vector.
 *  Si los tamaños son diferentes intersecta los tamaños y hace la copia en la
 *  intersección solamente. <br><b>Vector=VecSrc</b>.
 *  \param[in,out] Vector Es el vector a iniciar.
 *  \param[in] VecSrc Es el vector fuente de la copia.
 *  \return TRUE si todo fue bien o FALSE si no.
 *  \ingroup PdsCVectorGroup
 */
int pds_cvector_init_cvector(PdsCVector *Vector,const PdsCVector *VecSrc);


/** \fn int pds_cvector_init_rvector(PdsCVector *Vector,const PdsVector *VecSrc)
 *  \brief Inicia la parte real del vector de tipo puntero PdsCVector con un 
 *  vector de tipo PdsVector.
 *  Si los tamaños son diferentes intersecta los tamaños y hace la copia en la
 *  intersección solamente. <br><b>Vector.Real=VecSrc</b>.
 *  \param[in,out] Vector Es el vector a iniciar la parte real.
 *  \param[in] VecSrc Es el vector fuente de la copia.
 *  \return TRUE si todo fue bien o FALSE si no.
 *  \ingroup PdsCVectorGroup
 */
int pds_cvector_init_rvector(PdsCVector *Vector,const PdsVector *VecSrc);


/** \fn int pds_cvector_init_ivector(PdsCVector *Vector,const PdsVector *VecSrc)
 *  \brief Inicia la parte imaginaria del vector de tipo puntero PdsCVector con un 
 *  vector de tipo PdsVector.
 *  Si los tamaños son diferentes intersecta los tamaños y hace la copia en la
 *  intersección solamente. <br><b>Vector.Imag=VecSrc</b>.
 *  \param[in,out] Vector Es el vector a iniciar la parte imaginaria.
 *  \param[in] VecSrc Es el vector fuente de la copia.
 *  \return TRUE si todo fue bien o FALSE si no.
 *  \ingroup PdsCVectorGroup
 */
int pds_cvector_init_ivector(PdsCVector *Vector,const PdsVector *VecSrc);


/** \fn int pds_cvector_printf(const PdsCVector *Vector)
 *  \brief Imprime en pantalla un vector de tipo puntero PdsCVector.
 *  \param[in] Vector El vector a imprimir en pantalla.
 *  \return TRUE si todo fue bien o FALSE si no.
 *  \ingroup PdsCVectorGroup
 */
int pds_cvector_printf(const PdsCVector *Vector);


/** \fn int pds_cvector_fprintf(const PdsCVector *Vector,FILE *fd)
 *  \brief Imprime en el archivo que apunta fd un vector de tipo puntero PdsCVector.
 *  \param[in] Vector El vector a imprimir en pantalla.
 *  \param[in,out] fd El puntero al archivo.
 *  \return TRUE si todo fue bien o FALSE si no.
 *  \ingroup PdsCVectorGroup
 */
int pds_cvector_fprintf(const PdsCVector *Vector,FILE *fd);


/** \fn int pds_cvector_get_complex(const PdsCVector *Vector, PdsCaNatural x, PdsComplex *m)
 *  \brief Devuelve el valor complejo en la posición (x) del vector Vector.
 *  (x) inicia con (0).
 *  \param[in] Vector El vector en consulta.
 *  \param[in] x Posición x, el primer valor de x es cero.
 *  \param[out] m El valor complejo en la posición (x), en caso de error por fuera de rango 
 *  (x) entonces carga 0+0i en m, en caso de error de vector nulo carga 0+0i en m.
 *  \return TRUE si todo fue bien o FALSE si no (ej: Vector==NULL). 
 *  \ingroup PdsCVectorGroup
 */
int pds_cvector_get_complex(const PdsCVector *Vector, PdsCaNatural x, PdsComplex *m);


/** \fn int pds_cvector_set_complex(PdsCVector *Vector, PdsCaNatural x, PdsComplex m)
 *  \brief Escribe el valor m en la posición (x) del vector Vector.
 *  (x) inicia con (0).
 *  \param[in,out] Vector El vector a escribir.
 *  \param[in] x Posición x, el primer valor de x es cero.
 *  \param[in] m El valor complejo en la posición (x), en caso de error por fuera de rango 
 *  (x) entonces no hace nada y no se considera como error.
 *  \return TRUE si todo fue bien o FALSE si no (ej: Vector==NULL). 
 *  \ingroup PdsCVectorGroup
 */
int pds_cvector_set_complex(PdsCVector *Vector, PdsCaNatural x, PdsComplex m);


/** \fn int pds_cvector_set_real(PdsCVector *Vector, PdsCaNatural x, PdsCaReal m)
 *  \brief Escribe el valor m solo en la parte real de la posición (x) del vector Vector.
 *  (x) inicia con (0).
 *  \param[in,out] Vector El vector a escribir.
 *  \param[in] x Posición x, el primer valor de x es cero.
 *  \param[in] m La parte real a escribir en la posición (x), en caso de error por fuera de rango 
 *  (x) entonces no hace nada y no se considera como error.
 *  \return TRUE si todo fue bien o FALSE si no (ej: Vector==NULL). 
 *  \ingroup PdsCVectorGroup
 */
int pds_cvector_set_real(PdsCVector *Vector, PdsCaNatural x, PdsCaReal m);


/** \fn int pds_cvector_set_imag(PdsCVector *Vector, PdsCaNatural x, PdsCaReal m)
 *  \brief Escribe el valor m solo en la parte imaginaria de la posición (x) del vector Vector.
 *  (x) inicia con (0).
 *  \param[in,out] Vector El vector a escribir.
 *  \param[in] x Posición x, el primer valor de x es cero.
 *  \param[in] m La parte imaginaria a escribir en la posición (x), en caso de error por fuera de rango 
 *  (x) entonces no hace nada y no se considera como error.
 *  \return TRUE si todo fue bien o FALSE si no (ej: Vector==NULL). 
 *  \ingroup PdsCVectorGroup
 */
int pds_cvector_set_imag(PdsCVector *Vector, PdsCaNatural x, PdsCaReal m);


/** \fn int pds_cvector_conjugate(PdsCVector *Vector)
 *  \brief Encuentra el conjugado del vector Vector. <br><b>Vector=CONJ(Vector)</b>.
 *  \param[in,out] Vector Es el vector a obtener el conjugado.
 *  \return TRUE si todo fue bien o FALSE si no, por ejemplo vector NULL o
 *  tamanhos distintos.
 *  \ingroup PdsCVectorGroup
 */
int pds_cvector_conjugate(PdsCVector *Vector);


/** \fn int pds_cvector_conjugate_cvector(const PdsCVector *Vector,PdsCVector *VecDest)
 *  \brief Encuentra el conjugado del vector Vector, y el resultado es copiado
 *  en el vector VecDest. <br><b>VecDest=CONJ(Vector)</b>.
 *  \param[in] Vector Es el vector a obtener el conjugado.
 *  \param[in,out] VecDest Es el conjugado del vector Vector.
 *  \return TRUE si todo fue bien o FALSE si no, por ejemplo vector NULL o
 *  tamanhos distintos.
 *  \ingroup PdsCVectorGroup
 */
int pds_cvector_conjugate_cvector(const PdsCVector *Vector,PdsCVector *VecDest);


/** \fn int pds_cvector_modulus(const PdsCVector *Vector,PdsVector *VecDest)
 *  \brief Encuentra el Modulo del vector Vector, y el resultado es copiado
 *  en el vector VecDest. <br><b>VecDest=|Vector|</b>.
 *  \param[in] Vector Es el vector a obtener el modulo.
 *  \param[in,out] VecDest Es el módulo del vector Vector.
 *  \return TRUE si todo fue bien o FALSE si no, por ejemplo vector NULL o
 *  tamanhos distintos.
 *  \ingroup PdsCVectorGroup
 */
int pds_cvector_modulus(const PdsCVector *Vector,PdsVector *VecDest);


/** \fn int pds_cvector_modulus2(const PdsCVector *Vector,PdsVector *VecDest)
 *  \brief Encuentra el Modulo^2 del vector Vector, y el resultado es copiado
 *  en el vector VecDest. <br><b>VecDest=|Vector|^2</b>.
 *  \param[in] Vector Es el vector a obtener el modulo^2.
 *  \param[in,out] VecDest Es el módulo^2 del vector Vector.
 *  \return TRUE si todo fue bien o FALSE si no, por ejemplo vector NULL o
 *  tamanhos distintos.
 *  \ingroup PdsCVectorGroup
 */
int pds_cvector_modulus2(const PdsCVector *Vector,PdsVector *VecDest);


/** \fn int pds_cvector_phase(const PdsCVector *Vector,PdsVector *VecDest)
 *  \brief Encuentra la fase de Z=X+Yi phase=atan2(Y,X) del vector Vector, y el 
 *  resultado es copiado en el vector VecDest. <br><b>VecDest=Phase{Vector}</b>.
 *  \param[in,out] Vector Es el vector a obtener la fase.
 *  \param[in] VecDest Es la fase del vector Vector.
 *  \return TRUE si todo fue bien o FALSE si no, por ejemplo vector NULL o
 *  tamanhos distintos.
 *  \ingroup PdsCVectorGroup
 */
int pds_cvector_phase(const PdsCVector *Vector,PdsVector *VecDest);


/** \fn void pds_cvector_free(PdsCVector *Vector)
 *  \brief Libera un vector de tipo puntero PdsCVector.
 *  \param[in,out] Vector El vector a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsCVectorGroup
 */
void pds_cvector_free(PdsCVector *Vector);


/** \fn void pds_cvector_destroy(PdsCVector **Vector)
 *  \brief Libera un vector de tipo puntero PdsCVector, y limpia el puntero con NULL.
 *  \param[in,out] Vector El vector a liberar y limpiar.
 *  \return No retorna valor.
 *  \ingroup PdsCVectorGroup
 */
void pds_cvector_destroy(PdsCVector **Vector);


/**
 * @}
 */

#ifdef __cplusplus
}
#endif 

#endif	/* __PDSCVECTOR_H__ */ 

